@@ -190,12 +190,15 @@ pulseaudio_mpris_tick_cb (gpointer user_data)
           g_signal_connect (player, "playback-status", G_CALLBACK (pulseaudio_mpris_player_update_cb), mpris);
           g_signal_connect (player, "metadata", G_CALLBACK (pulseaudio_mpris_player_metadata_cb), mpris);
 
-          g_hash_table_insert (mpris->players, players[i], player);
+          g_hash_table_insert (mpris->players, g_strdup (players[i]), player);
 
           pulseaudio_config_add_mpris_player (mpris->config, players[i]);
         }
     }
 
+  if (players != NULL)
+    g_strfreev (players);
+
   return TRUE;
 }
 
@@ -213,7 +216,8 @@ pulseaudio_mpris_get_player_snapshot (PulseaudioMpris  *mpris,
                                       gboolean         *can_pause,
                                       gboolean         *can_go_previous,
                                       gboolean         *can_go_next,
-                                      gboolean         *can_raise)
+                                      gboolean         *can_raise,
+                                      GList           **playlists)
 {
   PulseaudioMprisPlayer *player;
   player = PULSEAUDIO_MPRIS_PLAYER (g_hash_table_lookup (mpris->players, name));
@@ -233,6 +237,7 @@ pulseaudio_mpris_get_player_snapshot (PulseaudioMpris  *mpris,
           *can_go_previous    = pulseaudio_mpris_player_can_go_previous (player);
           *can_go_next        = pulseaudio_mpris_player_can_go_next (player);
           *can_raise          = pulseaudio_mpris_player_can_raise (player);
+          *playlists          = pulseaudio_mpris_player_get_playlists (player);
         }
       else
         {
@@ -247,6 +252,7 @@ pulseaudio_mpris_get_player_snapshot (PulseaudioMpris  *mpris,
           *can_go_previous    = FALSE;
           *can_go_next        = FALSE;
           *can_raise          = FALSE;
+          *playlists          = NULL;
         }
       if (*title == NULL || g_strcmp0 (*title, "") == 0)
         *title = g_strdup(pulseaudio_mpris_player_get_player_title (player));
@@ -312,6 +318,31 @@ pulseaudio_mpris_notify_any_player (PulseaudioMpris *mpris,
 
 
 
+gboolean
+pulseaudio_mpris_activate_playlist (PulseaudioMpris *mpris,
+                                    const gchar     *name,
+                                    const gchar     *playlist)
+{
+  PulseaudioMprisPlayer *player;
+
+  g_return_val_if_fail(IS_PULSEAUDIO_MPRIS(mpris), FALSE);
+
+  player = g_hash_table_lookup(mpris->players, name);
+
+  if (player != NULL)
+  {
+    if (pulseaudio_mpris_player_is_connected (player))
+    {
+      pulseaudio_mpris_player_activate_playlist (player, playlist);
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
+
+
 static void
 pulseaudio_mpris_init (PulseaudioMpris *mpris)
 {
@@ -331,6 +362,9 @@ pulseaudio_mpris_finalize (GObject *object)
   mpris->config            = NULL;
   mpris->dbus_connection   = NULL;
 
+  if (mpris->players != NULL)
+    g_hash_table_destroy (mpris->players);
+
   (*G_OBJECT_CLASS (pulseaudio_mpris_parent_class)->finalize) (object);
 }
 
@@ -357,7 +391,7 @@ pulseaudio_mpris_new (PulseaudioConfig *config)
 
   mpris->config = config;
   mpris->dbus_connection = gconnection;
-  mpris->players = g_hash_table_new (g_str_hash, g_str_equal);
+  mpris->players = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)g_free);
   mpris->player_timer_id = g_timeout_add_seconds (1, pulseaudio_mpris_tick_cb, mpris);
 
   return mpris;
